@@ -20,6 +20,7 @@ namespace FSpot {
 			set {
 				path = value;
 				Load ();
+				System.Console.WriteLine ("XXXXX after load");
 			}
 		}
 
@@ -29,85 +30,37 @@ namespace FSpot {
 
 			if (Directory.Exists (path)) {
 				DirectoryInfo info = new DirectoryInfo (path);
-
 				LoadItems (info.GetFiles ());
 			} else if (File.Exists (path)) {
-				items = new FileBrowsableItem [] { new FileBrowsableItem (path) };
-			} else {
-				items = new FileBrowsableItem [0];
+				list.Clear ();
+				Add (new FileBrowsableItem (path));
 			}
 		}
 	}
 
-	public class FileCollection : IBrowsableCollection {
-		protected FileBrowsableItem [] items;
-
-		protected FileCollection ()
+	public class FileCollection : PhotoList {
+		protected FileCollection () : base (new IBrowsableItem [0])
 		{
-
 		}
 
-		public FileCollection (FileInfo [] files)
+		public FileCollection (FileInfo [] files) : this ()
 		{
 			LoadItems (files);
 		}
 
-		public int Count {
-			get {
-				return items.Length;
-			}
-		}
-
-		public bool Contains (IBrowsableItem item)
+		protected void LoadItems (FileInfo [] files) 
 		{
-			return IndexOf (item) >= 0;
-		}
-
-		// IBrowsableCollection
-		public IBrowsableItem [] Items {
-			get {
-				return items;
-			}
-		}
-
-		public IBrowsableItem this [int index] {
-			get {
-				return items [index];
-			}
-		}
-
-		public event FSpot.IBrowsableCollectionChangedHandler Changed;
-		public event FSpot.IBrowsableCollectionItemsChangedHandler ItemsChanged;
-
-		public int IndexOf (IBrowsableItem item)
-		{
-			return Array.IndexOf (items, item);
-		}
-
-		public void MarkChanged (int num)
-		{
-			MarkChanged (new BrowsableArgs (num));
-		}
-
-		public void MarkChanged (BrowsableArgs args)
-		{
-			if (this.ItemsChanged != null)
-				this.ItemsChanged (this, args);
-		}
-
-		protected void LoadItems (FileInfo [] files) {
-			ArrayList images = new ArrayList ();
-
+			ArrayList items = new ArrayList ();
 			foreach (FileInfo f in files) {
 				if (FSpot.ImageFile.HasLoader (f.FullName)) {
 					Console.WriteLine (f.FullName);
-					images.Add (new FileBrowsableItem (f.FullName));
+					items.Add (new FileBrowsableItem (f.FullName));
 				}
 			}
-				
-			items = images.ToArray (typeof (FileBrowsableItem)) as FileBrowsableItem [];
-		}
 
+			list = items;
+			this.Reload ();
+		}
 	}
 
 	public class FileBrowsableItem : IBrowsableItem {
