@@ -246,13 +246,13 @@ namespace FSpot {
 							e = sub.Directory [0].Lookup (TagId.DateTimeOriginal);
 							
 							if (e != null)
-								return DirectoryEntry.DateTimeFromString (e.StringValue);
+								return DirectoryEntry.DateTimeFromString (e.StringValue).ToUniversalTime ();
 						}
 						
 						e = tiff.Directory.Lookup (TagId.DateTime);
 
 						if (e != null)
-							return DirectoryEntry.DateTimeFromString (e.StringValue);
+							return DirectoryEntry.DateTimeFromString (e.StringValue).ToUniversalTime ();
 						
 						return base.Date;
 #endif
