@@ -3,6 +3,7 @@ using Gtk;
 using GtkSharp;
 using Glade;
 using Gnome;
+using Mono.Posix;
 using System;
 using System.Text;
 
@@ -12,7 +13,6 @@ using System.Text.RegularExpressions;
 using System.Web.Mail;
 
 using FSpot;
-using FSpot.Mail;
 using LibGPhoto2;
 
 public class MainWindow {
@@ -54,6 +54,7 @@ public class MainWindow {
 
 	[Glade.Widget] MenuItem adjust_color;
 	[Glade.Widget] MenuItem sharpen;
+	[Glade.Widget] MenuItem adjust_time;
 
 	[Glade.Widget] MenuItem update_thumbnail;
 	[Glade.Widget] MenuItem delete_from_drive;
@@ -75,6 +76,10 @@ public class MainWindow {
 	[Glade.Widget] MenuItem zoom_in;
 	[Glade.Widget] MenuItem zoom_out;
 
+	[Glade.Widget] RadioMenuItem month;
+	[Glade.Widget] RadioMenuItem directory;
+	[Glade.Widget] CheckMenuItem reverse_order;
+
 	// Find
 	[Glade.Widget] MenuItem find_tag;
 	[Glade.Widget] CheckMenuItem find_untagged;
@@ -114,6 +119,9 @@ public class MainWindow {
 	MainSelection selection;
 	
 	FSpot.Delay slide_delay;
+
+	Widget rl_button;
+	Widget rr_button;
 	
 	string last_import_path;
 	ModeType view_mode;
@@ -192,19 +200,27 @@ public class MainWindow {
 
 		toolbar = new Gtk.Toolbar ();
 		toolbar_vbox.PackStart (toolbar);
-		GtkUtil.MakeToolbarButton (toolbar, "f-spot-rotate-270", new System.EventHandler (HandleRotate270Command));
-		GtkUtil.MakeToolbarButton (toolbar, "f-spot-rotate-90", new System.EventHandler (HandleRotate90Command));
+
+		rl_button = GtkUtil.MakeToolbarButton (toolbar, "f-spot-rotate-270", new System.EventHandler (HandleRotate270Command));
+		rr_button = GtkUtil.MakeToolbarButton (toolbar, "f-spot-rotate-90", new System.EventHandler (HandleRotate90Command));
 		toolbar.AppendSpace ();
 
 		// FIXME putting these two toggle buttons in a radio group would prevent
 		// the two toggle sounds from being emitted every time you switch modes
 		browse_button = GtkUtil.MakeToolbarToggleButton (toolbar, "f-spot-browse", 
 								 new System.EventHandler (HandleToggleViewBrowse)) as ToggleButton;
+		SetTip (browse_button, Catalog.GetString ("Browse many photos simultaneously"));
 		edit_button = GtkUtil.MakeToolbarToggleButton (toolbar, "f-spot-edit-image", 
 							       new System.EventHandler (HandleToggleViewPhoto)) as ToggleButton;
+		SetTip (edit_button, Catalog.GetString ("View and edit a photo"));
+
 		toolbar.AppendSpace ();
-		GtkUtil.MakeToolbarButton (toolbar, "f-spot-fullscreen", new System.EventHandler (HandleViewFullscreen));
-		GtkUtil.MakeToolbarButton (toolbar, "f-spot-slideshow", new System.EventHandler (HandleViewSlideShow));
+
+		Widget fs_button = GtkUtil.MakeToolbarButton (toolbar, "f-spot-fullscreen", new System.EventHandler (HandleViewFullscreen));
+		SetTip (fs_button, Catalog.GetString ("View photos fullscreen"));
+		
+		Widget ss_button = GtkUtil.MakeToolbarButton (toolbar, "f-spot-slideshow", new System.EventHandler (HandleViewSlideShow));
+		SetTip (ss_button, Catalog.GetString ("View photos in a slideshow"));
 
 		tag_selection_widget = new TagSelectionWidget (db.Tags);
 		tag_selection_scrolled.Add (tag_selection_widget);
@@ -233,6 +249,7 @@ public class MainWindow {
 		query.Changed += HandleQueryChanged;
 
 		db.Photos.ItemsChanged += HandleDbItemsChanged;
+		db.Tags.ItemsChanged += HandleTagsChanged;
 #if SHOW_CALENDAR
 		FSpot.SimpleCalendar cal = new FSpot.SimpleCalendar (query);
 		cal.DaySelected += HandleCalendarDaySelected;
@@ -240,9 +257,8 @@ public class MainWindow {
 #endif
 
 		group_selector = new FSpot.GroupSelector ();
-		FSpot.GroupAdaptor adaptor = new FSpot.TimeAdaptor (query);
+		group_selector.Adaptor = new FSpot.TimeAdaptor (query);
 
-		group_selector.Adaptor  = adaptor;
 		group_selector.ShowAll ();
 		
 		if (zoom_scale != null) {
@@ -320,7 +336,10 @@ public class MainWindow {
 		photo_view.DragDataReceived += HandlePhotoViewDragDataReceived;
 
 		view_notebook.SwitchPage += HandleViewNotebookSwitchPage;
-		adaptor.GlassSet += HandleAdaptorGlassSet;
+		group_selector.Adaptor.GlassSet += HandleAdaptorGlassSet;
+		group_selector.Adaptor.Changed += HandleAdaptorChanged;
+		LoadPreference (Preferences.GROUP_ADAPTOR);
+		LoadPreference (Preferences.GROUP_ADAPTOR_ORDER_ASC);
 
 		this.selection = new MainSelection (this);
 		this.selection.Changed += HandleSelectionChanged;
@@ -431,7 +450,12 @@ public class MainWindow {
 		}
 		
 		if (args is TimeChangedEventArgs)
-			InvalidateViews ();
+			query.RequestReload ();
+	}
+
+	private void HandleTagsChanged (object sender, DbItemEventArgs args)
+	{
+		UpdateTagEntryFromSelection ();
 	}
 
 	void HandleViewNotebookSwitchPage (object sender, SwitchPageArgs args)
@@ -850,6 +874,11 @@ public class MainWindow {
 		JumpTo (index);
 	}
 
+	void HandleAdaptorChanged (FSpot.GroupAdaptor sender)
+	{
+		UpdateGlass ();
+	}
+
 	/*
 	 * Keep the glass temporal slider in sync with the user's scrolling in the icon_view
 	 */
@@ -860,7 +889,7 @@ public class MainWindow {
 			return;
 
 		int cell_num = icon_view.TopLeftVisibleCell();
-		if (cell_num == -1 || cell_num == lastTopLeftCell)
+		if (cell_num == -1 /*|| cell_num == lastTopLeftCell*/)
 			return;
 
 		lastTopLeftCell = cell_num;
@@ -886,7 +915,7 @@ public class MainWindow {
 
 	void HandleIconViewReady (object sender, EventArgs args)
 	{
-		LoadPreference (Preferences.ICON_VIEW_POSITION);
+		LoadPreference (Preferences.GLASS_POSITION);
 
 		// We only want to set the position the first time
 		// the icon_view is ready (eg on startup)
@@ -998,6 +1027,27 @@ public class MainWindow {
 		}
 	}
 
+#if false
+	public void ImportUdi (string udi)
+	{
+		/* probably a camera we need to contruct on of our gphoto2 uris */
+		Hal.Device dev = new Hal.Device (Core.HalContext, udi);
+		string mount_point = dev.GetPropertyString ("volume.mount_point");
+		int bus = dev.GetPropertyInt ("usb.bus_number");
+		int device = dev.GetPropertyInt ("usb.linux.device_number");
+		System.Console.WriteLine ("dev = {1} exists = {2} mount_point = {0} {3},{4}", mount_point, dev, dev.Exists, bus, device);
+
+		if (! dev.Exists || mount_point != null) {
+			ImportFile (mount_point);
+		} else {
+			string gphoto_uri = String.Format ("gphoto2:usb:{0},{1}", bus.ToString ("d3") , device.ToString ("d3"));
+			System.Console.WriteLine ("gphoto_uri = {0}", gphoto_uri);
+			ImportCamera (gphoto_uri);
+		} 
+			
+	}
+#endif
+
 	void HandleIconViewDragDataReceived (object sender, DragDataReceivedArgs args)
 	{
 	 	Widget source = Gtk.Drag.GetSourceWidget (args.Context);     
@@ -1173,7 +1223,7 @@ public class MainWindow {
 	{
 		db.Sync = false;
 		ImportCommand command = new ImportCommand (main_window);
-		if (command.ImportFromFile (db.Photos, this.last_import_path) > 0) {
+		if (command.ImportFromFile (db.Photos, null) > 0) {
 			this.last_import_path = command.ImportPath;
 			UpdateQuery ();
 		}
@@ -1347,7 +1397,7 @@ public class MainWindow {
 		GnomeUtil.UrlShow (main_window, url.ToString ());
 	}
 
-	void HandleAbout (object sender, EventArgs args)
+	public static void HandleAbout (object sender, EventArgs args)
 	{
 		string [] authors = new string [] {
 			"Ettore Perazzoli",
@@ -1393,26 +1443,57 @@ public class MainWindow {
 
                 new About ("F-Spot", 
 			   FSpot.Defines.VERSION, 
-			   "Copyright 2003-2005 Novell Inc.",
+			   Mono.Posix.Catalog.GetString ("Copyright 2003-2006 Novell Inc."),
                            null, authors, new string [0], translators, null).Show();
 	}
 
-	void HandleArrangeByTime (object sender, EventArgs args)
+	public void HandleArrangeByTime (object sender, EventArgs args)
 	{
+		if (group_selector.Adaptor is TimeAdaptor)
+			return;
+
 		group_selector.Adaptor.GlassSet -= HandleAdaptorGlassSet;
-		FSpot.GroupAdaptor adaptor = new FSpot.TimeAdaptor (query);
-		group_selector.Adaptor = adaptor;
+		group_selector.Adaptor.Changed -= HandleAdaptorChanged;
+		group_selector.Adaptor = new FSpot.TimeAdaptor (query);
+
 		group_selector.Mode = FSpot.GroupSelector.RangeType.Min;
-		adaptor.GlassSet += HandleAdaptorGlassSet;
+		group_selector.Adaptor.GlassSet += HandleAdaptorGlassSet;
+		group_selector.Adaptor.Changed += HandleAdaptorChanged;
+
+		if (sender != month)
+			month.Active = true;
 	}
 
-	void HandleArrangeByDirectory (object sender, EventArgs args)
+	public void HandleArrangeByDirectory (object sender, EventArgs args)
 	{
+		if (group_selector.Adaptor is DirectoryAdaptor)
+			return;
+
 		group_selector.Adaptor.GlassSet -= HandleAdaptorGlassSet;
-		FSpot.GroupAdaptor adaptor = new FSpot.DirectoryAdaptor (query);		
-		group_selector.Adaptor = adaptor;
+		group_selector.Adaptor.Changed -= HandleAdaptorChanged;
+		group_selector.Adaptor = new FSpot.DirectoryAdaptor (query); 	
+
 		group_selector.Mode = FSpot.GroupSelector.RangeType.Min;
-		adaptor.GlassSet += HandleAdaptorGlassSet;
+		group_selector.Adaptor.GlassSet += HandleAdaptorGlassSet;
+		group_selector.Adaptor.Changed += HandleAdaptorChanged;
+
+		if (sender != directory)
+			directory.Active = true;
+	}
+	
+	public void HandleReverseOrder (object sender, EventArgs args)
+	{
+		Gtk.CheckMenuItem item = sender as Gtk.CheckMenuItem;
+
+		if (group_selector.Adaptor.OrderAscending == item.Active)
+			return;
+		
+		group_selector.Adaptor.OrderAscending = item.Active;
+		icon_view.QueueDraw ();
+
+		// FIXME this is blah...we need UIManager love here
+		if (item != reverse_order)
+			reverse_order.Active = item.Active;
 	}
 
 	// Called when the user clicks the X button	
@@ -1449,11 +1530,13 @@ public class MainWindow {
 		Preferences.Set (Preferences.SHOW_TAGS,			icon_view.DisplayTags);
 		Preferences.Set (Preferences.SHOW_DATES,		icon_view.DisplayDates);
 
+		Preferences.Set (Preferences.GROUP_ADAPTOR,		(group_selector.Adaptor is DirectoryAdaptor) ? 1 : 0);
+		Preferences.Set (Preferences.GROUP_ADAPTOR_ORDER_ASC,   group_selector.Adaptor.OrderAscending);
+		Preferences.Set (Preferences.GLASS_POSITION,		group_selector.GlassPosition);
+		
 		Preferences.Set (Preferences.SIDEBAR_POSITION,		main_hpaned.Position);
 		Preferences.Set (Preferences.ZOOM,			icon_view.Zoom);
 	
-		Preferences.Set (Preferences.ICON_VIEW_POSITION, icon_view.TopLeftVisibleCell ());
-		
 		tag_selection_widget.SaveExpandDefaults ();
 
 		this.Window.Destroy ();
@@ -1602,7 +1685,7 @@ public class MainWindow {
 
 		string msg = Mono.Posix.Catalog.GetString("This operation will merge the selected tags and any sub-tags into a single tag.");
 
-		string ok_caption = Mono.Posix.Catalog.GetString ("_Merge tags");
+		string ok_caption = Mono.Posix.Catalog.GetString ("_Merge Tags");
 		
 		if (ResponseType.Ok != HigMessageDialog.RunHigConfirmation(main_window, 
 									   DialogFlags.DestroyWithParent, 
@@ -1640,12 +1723,14 @@ public class MainWindow {
 
 	void HandleAdjustTime (object sender, EventArgs args)
 	{
-		new TimeDialog (db, new PhotoList (Selection.Items));
+		PhotoList list = new PhotoList (Selection.Items);
+		list.Sort (new Photo.CompareDateName ());
+		new TimeDialog (db, list);
 	}
 
 	void HandleAdjustColor (object sender, EventArgs args)
 	{
-		if (ActiveIndex () > 0) {
+		if (ActiveIndex () >= 0) {
 			SetViewMode (ModeType.PhotoView);
 			new FSpot.ColorDialog (photo_view.View);
 		}
@@ -1933,7 +2018,7 @@ public class MainWindow {
 				fname).Replace ("_", "__");
 		else
 			msg = String.Format (
-				Mono.Posix.Catalog.GetString ("An error of type {0} ocurred when deleting the file:\n{1}"),
+				Mono.Posix.Catalog.GetString ("An error of type {0} occurred while deleting the file:\n{1}"),
 				e.GetType (), fname.Replace ("_", "__"));
 		
 		HigMessageDialog.RunHigConfirmation (
@@ -2217,6 +2302,25 @@ public class MainWindow {
 				//display_dates_menu_item.Toggle ();
 			break;
 		
+		case Preferences.GROUP_ADAPTOR:
+			if ((int) val == 1)
+				directory.Active = true;
+			break;
+
+		case Preferences.GROUP_ADAPTOR_ORDER_ASC:
+			if (reverse_order.Active != (bool) val)
+				reverse_order.Active = (bool) val;
+			//group_selector.Adaptor.OrderAscending = (bool) val;
+			break;
+
+		case Preferences.GLASS_POSITION:
+			if (query.Count > 0) {
+				IBrowsableItem photo = group_selector.Adaptor.PhotoFromIndex ((int) val);
+				
+				if (photo != null)
+					JumpTo (query.IndexOf (photo));
+			}
+			break;
 		case Preferences.SIDEBAR_POSITION:
 			if (main_hpaned.Position != (int) val)
 				main_hpaned.Position = (int) val;
@@ -2226,13 +2330,6 @@ public class MainWindow {
 			icon_view.Zoom = (double) val;
 			break;
 		
-		case Preferences.ICON_VIEW_POSITION:
-			if (icon_view.TopLeftVisibleCell () != (int) val) {
-				icon_view.FocusCell = (int) val;
-				photo_view.Item.Index = (int) val;
-				icon_view.ScrollTo ((int) val, false);
-			}
-			break;
 		case Preferences.METADATA_EMBED_IN_IMAGE:
 			write_metadata = (bool) val;
 			break;
@@ -2317,6 +2414,7 @@ public class MainWindow {
 
 		set_as_background.Sensitive = single_active;
 		adjust_color.Sensitive = single_active;
+		adjust_time.Sensitive = active_selection;
 
 		attach_tag.Sensitive = active_selection;
 		remove_tag.Sensitive = active_selection;
@@ -2338,8 +2436,84 @@ public class MainWindow {
 		delete_selected_tag.Sensitive = tag_sensitive;
 		edit_selected_tag.Sensitive = tag_sensitive;
 
+
 		attach_tag_to_selection.Sensitive = tag_sensitive && active_selection;
 		remove_tag_from_selection.Sensitive = tag_sensitive && active_selection;
+		
+		if (rl_button != null) {
+			if (selection.Count == 0) {
+				rl_button.Sensitive = false;
+				SetTip (rl_button, "");
+			} else {
+				rl_button.Sensitive = true;
+
+				string msg = Catalog.GetPluralString ("Rotate selected photo left",
+				    "Rotate selected photos left", selection.Count);
+				SetTip (rl_button, String.Format (msg, selection.Count));
+			}
+		}
+		
+		if (rr_button != null) {
+			if (selection.Count == 0) {
+				rr_button.Sensitive = false;
+				SetTip (rr_button, "");
+			} else {
+				rr_button.Sensitive = true;
+
+				string msg = Catalog.GetPluralString ("Rotate selected photo right",
+				    "Rotate selected photos right", selection.Count);
+				SetTip (rr_button, String.Format (msg, selection.Count));
+			}
+		}
+	}
+
+	public void HandleOpenWith (Gnome.Vfs.MimeApplication mime_application)
+	{
+		Photo[] selected = SelectedPhotos ();
+
+		if (selected == null || selected.Length < 1)
+			return;
+		
+		string header = Catalog.GetPluralString ("Create New Version?", "Create New Versions?", selected.Length); 
+		
+		string msg = String.Format (Catalog.GetPluralString (
+				"Before launching {1}, should F-Spot create a new version of the selected photo to preserve the original?",
+				"Before launching {1}, should F-Spot create new versions of the selected photos to preserve the originals?", selected.Length),
+				selected.Length, mime_application.Name);
+
+		// FIXME add cancel button? add help button?
+		HigMessageDialog hmd = new HigMessageDialog(main_window, DialogFlags.DestroyWithParent, 
+								   MessageType.Question, Gtk.ButtonsType.None,
+								   header, msg);
+		hmd.AddButton (Gtk.Stock.No, Gtk.ResponseType.No, false);
+		//hmd.AddButton (Gtk.Stock.Cancel, Gtk.ResponseType.Cancel, false);
+		hmd.AddButton (Gtk.Stock.Yes, Gtk.ResponseType.Yes, true);
+
+		Gtk.ResponseType response = Gtk.ResponseType.Cancel;
+ 		try {
+ 			response = (Gtk.ResponseType) hmd.Run();
+ 		} finally {
+ 			hmd.Destroy();
+ 		}
+
+		if (response == Gtk.ResponseType.Cancel)
+			return;
+
+		bool create_new_versions = (response == Gtk.ResponseType.Yes);
+
+		GLib.List uri_list = new GLib.List (typeof (string));
+		foreach (Photo photo in selected) {
+			if (create_new_versions) {
+				// exception handling? Out of space, blah, blah
+				uint version = photo.CreateNamedVersion (mime_application.Name, photo.DefaultVersionId, true);
+				photo.DefaultVersionId = version;
+				UpdateForVersionIdChange (version);
+			}
+
+			uri_list.Append (photo.DefaultVersionPath);
+		}
+		
+		mime_application.Launch (uri_list);
 	}
 
 	// Tag typing
@@ -2616,8 +2790,22 @@ public class MainWindow {
 		return false;
 	}
 
+	public string [] SelectedMimeTypes ()
+	{
+		ArrayList mimes = new ArrayList ();
+
+		foreach (Photo p in SelectedPhotos ()) {
+			string mime = Gnome.Vfs.MimeType.GetMimeTypeForUri (p.DefaultVersionPath);
+
+			if (! mimes.Contains (mime))
+				mimes.Add (mime);
+		}
+
+		return mimes.ToArray (typeof (string)) as string [];
+	}
+
 	public static void SetTip (Widget widget, string tip)
 	{
-		toolTips.SetTip (widget, tip, tip);
+		toolTips.SetTip (widget, tip, null);
 	}
 }
