@@ -239,6 +239,34 @@ namespace FSpot {
 			Add (sink, new Entity (""), predicate, type, values);
 		}
 
+		public void Update (string predicate, string type, string [] values)
+		{
+			Entity anon = null;
+
+			System.Collections.ArrayList to_remove = new System.Collections.ArrayList ();
+			foreach (Statement stmt in this) {
+				if (stmt.Predicate == MetadataStore.Namespaces.Resolve (predicate)) {
+					anon = (Entity) stmt.Object;
+					to_remove.Add (stmt);
+					break;
+				}
+			}
+
+			if (anon != null) {
+				foreach (Statement stmt in this) {
+					if (stmt.Subject == anon) {
+						to_remove.Add (stmt);
+					}
+				}
+			}
+
+			foreach (Statement stmt in to_remove)
+				this.Remove (stmt);
+
+			if (values.Length > 0)
+				Add (this, predicate, type, values);
+		}
+		
 		public static void Add (StatementSink sink, Entity subject, string predicate, string type, string [] values)
 		{
 			if (values == null) {
