@@ -1,26 +1,26 @@
+using System;
 
 namespace FSpot {
 	public class PhotoImageView : ImageView {
-		public static double ZoomMultipler = 1.1;
-
 		public delegate void PhotoChangedHandler (PhotoImageView view);
 		public event PhotoChangedHandler PhotoChanged;
 		
 		protected BrowsablePointer item;
-		
+		protected FSpot.Loupe loupe;
+		protected FSpot.Loupe sharpener;
+
 		public PhotoImageView (IBrowsableCollection query)
 		{
 			loader = new FSpot.AsyncPixbufLoader ();
 			loader.AreaUpdated += HandlePixbufAreaUpdated;
 			loader.AreaPrepared += HandlePixbufPrepared;
 			loader.Done += HandleDone;
-
+			
 			this.SizeAllocated += HandleSizeAllocated;
 			this.KeyPressEvent += HandleKeyPressEvent;
 			this.ScrollEvent += HandleScrollEvent;
-			query.ItemsChanged += HandleQueryItemsChanged;
 			this.item = new BrowsablePointer (query, -1);
-			item.IndexChanged += PhotoIndexChanged;
+			item.Changed += PhotoItemChanged;
 			this.Destroyed += HandleDestroyed;
 			this.SetTransparentColor (this.Style.BaseColors [(int)Gtk.StateType.Normal]);
 		}
@@ -66,72 +66,27 @@ namespace FSpot {
 			if (!Item.IsValid)
 				return;
 			
-			PhotoIndexChanged (Item, null);
-			
-		}
-		/*
-		private void HandleQueryChanged (IBrowsableCollection browsable)
-		{
-			if (query == browsable)
-				Reload ();
-		}
-		*/
-
-		public void HandleQueryItemsChanged (IBrowsableCollection browsable, BrowsableArgs args)
-		{
-			foreach (int item in args.Items) {
-				if (item == Item.Index)
-					Reload ();
-			}
+			PhotoItemChanged (Item, null);
 		}
 
 		// Display.
-		private void HandlePixbufAreaUpdated (object sender, Gdk.Rectangle area)
+		private void HandlePixbufAreaUpdated (object sender, AreaUpdatedArgs args)
 		{
-			area = this.ImageCoordsToWindow (area);
+			Gdk.Rectangle area = this.ImageCoordsToWindow (args.Area);
 			this.QueueDrawArea (area.X, area.Y, area.Width, area.Height);
 		}
 		
-		private void HandlePixbufPrepared (object sender, System.EventArgs args)
+		private void HandlePixbufPrepared (object sender, AreaPreparedArgs args)
 		{
 			Gdk.Pixbuf prev = this.Pixbuf;
 			Gdk.Pixbuf next = loader.Pixbuf;
 
-#if SPEED_COPY_DATA
-			if (next != null && prev != null && next.Width == prev.Width && prev.Height == next.Height)
-				prev.CopyArea (0, 0, next.Width, next.Height, next, 0, 0);
-			else
-				next.Fill (0x00000000);
-#endif
-#if true
-			System.Uri uri = Item.Current.DefaultVersionUri;
-			try {
-				Gdk.Pixbuf thumb = new Gdk.Pixbuf (ThumbnailGenerator.ThumbnailPath (uri));
-				if (thumb != null && next != null)
-					thumb.Composite (next, 0, 0,
-							 next.Width, next.Height,
-							 0.0, 0.0,
-							 next.Width/(double)thumb.Width, next.Height/(double)thumb.Height,
-							 Gdk.InterpType.Bilinear, 0xff);
-				
-				if (thumb != null) {
-					if (!ThumbnailGenerator.ThumbnailIsValid (thumb, uri))
-						FSpot.ThumbnailGenerator.Default.Request (uri.LocalPath, 0, 256, 256);
-					
-					thumb.Dispose ();
-				}
-			} catch (System.Exception e) {
-				FSpot.ThumbnailGenerator.Default.Request (uri.LocalPath, 0, 256, 256);	
-				if (!(e is GLib.GException)) 
-					System.Console.WriteLine (e.ToString ());
-			}
-#endif
-
 			this.Pixbuf = next;
 			if (prev != null)
 				prev.Dispose ();
 
-			this.ZoomFit ();
+			UpdateMinZoom ();
+			this.ZoomFit (args.ReducedResolution);
 		}
 
 		private void HandleDone (object sender, System.EventArgs args)
@@ -161,27 +116,31 @@ namespace FSpot {
 								      PixbufUtils.ErrorPixbuf.Width, 
 								      PixbufUtils.ErrorPixbuf.Height);
 
+				UpdateMinZoom ();
 				this.ZoomFit ();
 			} else {
 				this.Pixbuf = loader.Pixbuf;
 
-				if (!loader.Prepared)
+				if (!loader.Prepared) {
+					UpdateMinZoom ();
 					this.ZoomFit ();
+				}
 			}
 
 			if (prev != this.Pixbuf && prev != null)
 				prev.Dispose ();
-		}		
+		}
 		
 		private bool fit = true;
 		public bool Fit {
 			get {
-				return fit;
+				return (Zoom == MIN_ZOOM);
 			}
 			set {
-				fit = value;
-				if (fit)
+				if (!fit && value)
 					ZoomFit ();
+				
+				fit = value;
 			}
 		}
 
@@ -194,8 +153,33 @@ namespace FSpot {
 			}
 			
 			set {
-				this.Fit = false;
-				this.SetZoom (value, value);
+				//Console.WriteLine ("Setting zoom to {0}, MIN = {1}", value, MIN_ZOOM);
+				value = System.Math.Min (value, MAX_ZOOM);
+				value = System.Math.Max (value, MIN_ZOOM);
+
+				double zoom = Zoom;
+				if (value == zoom)
+					return;
+
+				if (System.Math.Abs (zoom - value) < System.Double.Epsilon)
+					return;
+
+				if (value == MIN_ZOOM)
+					this.Fit = true;
+				else {
+					this.Fit = false;
+					this.SetZoom (value, value);
+				}
+			}
+		}
+		
+		// Zoom scaled between 0.0 and 1.0
+		public double NormalizedZoom {
+			get {
+				return (Zoom - MIN_ZOOM) / (MAX_ZOOM - MIN_ZOOM);
+			}
+			set {
+				Zoom = (value * (MAX_ZOOM - MIN_ZOOM)) + MIN_ZOOM;
 			}
 		}
 		
@@ -203,11 +187,10 @@ namespace FSpot {
 		{
 			if (fit)
 				ZoomFit ();
-		}	
+		}
 
 		bool load_async = true;
 		FSpot.AsyncPixbufLoader loader;
-		FSpot.AsyncPixbufLoader next_loader;
 
 		private void LoadErrorImage (System.Exception e)
 		{
@@ -222,26 +205,34 @@ namespace FSpot {
 			if (old != null)
 				old.Dispose ();
 			
+			UpdateMinZoom ();
 			this.ZoomFit ();
 		}
 
-		private void PhotoIndexChanged (BrowsablePointer item, IBrowsableItem old_item) 
+		private void PhotoItemChanged (BrowsablePointer item, BrowsablePointerChangedArgs args) 
 		{
 			// If it is just the position that changed fall out
-			if (old_item != null && Item.IsValid && this.Item.Current.DefaultVersionUri == old_item.DefaultVersionUri)
+			if (args != null && 
+			    args.PreviousItem != null &&
+			    Item.IsValid && 
+			    this.Item.Current.DefaultVersionUri == args.PreviousItem.DefaultVersionUri)
 				return;
 
 			if (load_async) {
+				Gdk.Pixbuf old = this.Pixbuf;
 				try {
-					if (Item.IsValid)
-						loader.Load (Item.Current.DefaultVersionUri.LocalPath);
-					else
+					if (Item.IsValid) {
+						System.Uri uri = Item.Current.DefaultVersionUri;
+						loader.Load (uri);
+					} else
 						LoadErrorImage (null);
 
 				} catch (System.Exception e) {
 					System.Console.WriteLine (e.ToString ());
 					LoadErrorImage (e);
 				}
+				if (old != null)
+					old.Dispose ();
 			} else {	
 				Gdk.Pixbuf old = this.Pixbuf;
 				this.Pixbuf = FSpot.PhotoLoader.Load (item.Collection, 
@@ -249,6 +240,7 @@ namespace FSpot {
 				if (old != null)
 					old.Dispose ();
 
+				UpdateMinZoom ();
 				this.ZoomFit ();
 			}
 			
@@ -257,11 +249,28 @@ namespace FSpot {
 			if (PhotoChanged != null)
 				PhotoChanged (this);
 		}
-
+		
+		public void ZoomIn ()
+		{
+			Zoom = Zoom * ZOOM_FACTOR;
+		}
+		
+		public void ZoomOut ()
+		{
+			Zoom = Zoom / ZOOM_FACTOR;
+		}
+		
+		bool upscale;
 		private void ZoomFit ()
 		{
+			ZoomFit (upscale);
+		}
+
+		private void ZoomFit (bool upscale)
+		{			
 			Gdk.Pixbuf pixbuf = this.Pixbuf;
 			Gtk.ScrolledWindow scrolled = this.Parent as Gtk.ScrolledWindow;
+			this.upscale = upscale;
 			
 			if (pixbuf == null)
 				return;
@@ -273,7 +282,7 @@ namespace FSpot {
 								   (uint) available_height,
 								   (uint) pixbuf.Width, 
 								   (uint) pixbuf.Height, 
-								   false);
+								   upscale);
 			
 			double image_zoom = zoom_to_fit;
 			/*
@@ -290,20 +299,46 @@ namespace FSpot {
 			if (scrolled != null)
 				scrolled.SetPolicy (Gtk.PolicyType.Automatic, Gtk.PolicyType.Automatic);
 		}
+		
+		private void HandleLoupeDestroy (object sender, EventArgs args)
+		{
+			if (sender == loupe)
+				loupe = null;
+
+			if (sender == sharpener)
+				sharpener = null;
+
+		}
 
 		[GLib.ConnectBefore]
 		private void HandleKeyPressEvent (object sender, Gtk.KeyPressEventArgs args)
 		{
 			// FIXME I really need to figure out why overriding is not working
 			// for any of the default handlers.
+			args.RetVal = true;
+		
+			// Check for KeyPad arrow keys, which scroll the window when zoomed in
+			// but should go to the next/previous photo when not zoomed (no scrollbars)
+			if (this.Fit) {
+				switch (args.Event.Key) {
+				case Gdk.Key.Up:
+				case Gdk.Key.Left:
+				case Gdk.Key.KP_Up:
+				case Gdk.Key.KP_Left:
+					this.Item.MovePrevious ();
+					return;
+				case Gdk.Key.Down:
+				case Gdk.Key.Right:
+				case Gdk.Key.KP_Down:
+				case Gdk.Key.KP_Right:
+					this.Item.MoveNext ();
+					return;
+				}
+			}
 
 			switch (args.Event.Key) {
 			case Gdk.Key.Page_Up:
 			case Gdk.Key.KP_Page_Up:
-			case Gdk.Key.Up:
-			case Gdk.Key.KP_Up:
-			case Gdk.Key.Left:
-			case Gdk.Key.KP_Left:
 				this.Item.MovePrevious ();
 				break;
 			case Gdk.Key.Home:
@@ -316,10 +351,6 @@ namespace FSpot {
 				break;
 			case Gdk.Key.Page_Down:
 			case Gdk.Key.KP_Page_Down:
-			case Gdk.Key.Down:
-			case Gdk.Key.KP_Down:
-			case Gdk.Key.Right:
-			case Gdk.Key.KP_Right:
 			case Gdk.Key.space:
 			case Gdk.Key.KP_Space:
 				this.Item.MoveNext ();
@@ -338,17 +369,34 @@ namespace FSpot {
 				break;
 			case Gdk.Key.minus:
 			case Gdk.Key.KP_Subtract:
-				this.Zoom /= ZoomMultipler;
+				ZoomOut ();
 				break;
+			case Gdk.Key.s:
+				if (sharpener == null) {
+					sharpener = new Sharpener (this);
+					sharpener.Destroyed += HandleLoupeDestroy;
+				}
+
+				sharpener.Show ();
+				break;
+			case Gdk.Key.v:
+				if (loupe == null) {
+					loupe = new Loupe (this);
+					loupe.Destroyed += HandleLoupeDestroy;
+				}
+
+				loupe.Show ();
+				break;
+			case Gdk.Key.equal:
 			case Gdk.Key.plus:
 			case Gdk.Key.KP_Add:
-				this.Zoom *= ZoomMultipler;
+				ZoomIn ();
 				break;
 			default:
 				args.RetVal = false;
 				return;
 			}
-			args.RetVal = true;
+
 			return;
 		}
 		
