@@ -103,6 +103,21 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 		return string.Compare (photo1.name, photo2.name);
 	}
 
+	public class CompareDateName : IComparer {
+		public int Compare (object obj1, object obj2) {
+			Photo p1 = (Photo)obj1;
+			Photo p2 = (Photo)obj2;
+
+			int result = Photo.CompareDate (p1, p2);
+			
+			if (result == 0)
+				result = CompareName (p1, p2);
+
+			return result;
+		}
+
+	}
+
 	public class CompareDirectory : IComparer {
 		public int Compare (object obj1, object obj2) {
 			Photo p1 = (Photo)obj1;
@@ -117,6 +132,14 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 		}
 	}
 
+	public class RandomSort : IComparer {
+		Random random = new Random ();
+		
+		public int Compare (object obj1, object obj2) {
+			return random.Next (-5, 5);
+		}
+	}
+
 	// The time is always in UTC.
 	private DateTime time;
 	public DateTime Time {
@@ -346,6 +369,26 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 		}
 	}
 
+	public uint CreateNamedVersion (string name, uint base_version_id, bool create_file)
+	{
+		int num = 1;
+		
+		string final_name;
+		while (true) {
+			final_name = String.Format (
+					Mono.Posix.Catalog.GetPluralString ("Modified in {1}", "Modified in {1} ({0})", num),
+					num, name);
+
+			if (num > 1)
+				final_name = name + String.Format(" ({0})", num);
+
+			if (! VersionNameExists (final_name))
+				return CreateVersion (final_name, base_version_id, create_file);
+
+			num ++;
+		}
+	}
+
 	public void RenameVersion (uint version_id, string new_name)
 	{
 		if (version_id == OriginalVersionId)
@@ -455,7 +498,7 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 			FSpot.JpegFile jimg = img as FSpot.JpegFile;
 			
 			jimg.SetDescription (this.Description);
-			jimg.SetDateTimeOriginal (this.Time);
+			jimg.SetDateTimeOriginal (this.Time.ToLocalTime ());
 			jimg.SetXmp (UpdateXmp (this, jimg.Header.GetXmp ()));
 
 			jimg.SaveMetaData (path);
@@ -475,22 +518,26 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 	{
 		uint version = DefaultVersionId;
 		ImageFile img = ImageFile.Create (DefaultVersionUri.LocalPath);
-		Exif.ExifData exif_data = new Exif.ExifData (DefaultVersionPath);
 
 		// Always create a version if the source is not a jpeg for now.
 		create_version = create_version || !(img is FSpot.JpegFile);
 
+		if (buffer == null)
+			throw new ApplicationException ("invalid (null) image");
+
 		if (create_version)
 			version = CreateDefaultModifiedVersion (DefaultVersionId, false);
 
 		try {
 			string version_path = GetVersionPath (version);
 			
-			PixbufUtils.SaveJpeg (buffer, version_path, 95, exif_data);
+			using (Stream stream = File.OpenWrite (version_path)) {
+				img.Save (buffer, stream);
+			}
 			FSpot.ThumbnailGenerator.Create (version_path).Dispose ();
 			DefaultVersionId = version;
-			
 		} catch (System.Exception e) {
+			System.Console.WriteLine (e);
 			if (create_version)
 				DeleteVersion (version);
 			
