@@ -315,7 +315,7 @@ public class PhotoView : EventBox {
 							   area);
 		} else { // Crop (I told you it was ugly)
 			edited = new Pixbuf (original_pixbuf.Colorspace, 
-					     false, original_pixbuf.BitsPerSample,
+					     original_pixbuf.HasAlpha, original_pixbuf.BitsPerSample,
 					     width, height);
 			
 			original_pixbuf.CopyArea (x, y, width, height, edited, 0, 0);
@@ -471,6 +471,7 @@ public class PhotoView : EventBox {
 		tag_view = new TagView (tag_view_box);
 		tag_view_box.Add (tag_view);
 		inner_hbox.PackStart (tag_view_box, false, true, 0);
+		FSpot.Global.ModifyColors (tag_view);
 
 		Label comment = new Label (Catalog.GetString ("Comment:"));
 		inner_hbox.PackStart (comment, false, false, 0);
@@ -498,7 +499,7 @@ public class PhotoView : EventBox {
 		redeye_button.Clicked += new EventHandler (HandleRedEyeButtonClicked);
 
 		color_button = new ToolbarButton ();
-		color_button.Add (new Gtk.Image ("f-spot-edit-image", IconSize.Button));
+		color_button.Add (new Gtk.Image ("f-spot-adjust-colors", IconSize.Button));
 		toolbar_hbox.PackStart (color_button, false, true, 0);
 	
 		color_button.Clicked += new EventHandler (HandleColorButtonClicked);
@@ -541,6 +542,8 @@ public class PhotoView : EventBox {
 		tips.SetTip (constraints_option_menu, Mono.Posix.Catalog.GetString ("Constrain the aspect ratio of the selection"), "");
 		tips.SetTip (display_next_button, Mono.Posix.Catalog.GetString ("Next photo"), "");
 		tips.SetTip (display_previous_button, Mono.Posix.Catalog.GetString ("Previous photo"), "");
+		tips.SetTip (desaturate_button, Mono.Posix.Catalog.GetString ("Convert the photo to black and white"), "");
+		tips.SetTip (sepia_button, Mono.Posix.Catalog.GetString ("Convert the photo to sepia tones"), "");
 	}
 
 }
