@@ -1,11 +1,31 @@
 using ICSharpCode.SharpZipLib.Zip.Compression;
 using SemWeb;
 using Cms;
+using System.IO;
+using FSpot.Xmp;
+using System.Collections;
 
 namespace FSpot.Png {
 	public class PngFile : ImageFile, SemWeb.StatementSource {
-		System.Collections.ArrayList chunk_list;
-		
+		PngHeader header;
+
+		private PngHeader Header {
+			get {
+				if (header == null) {
+					using (System.IO.Stream input = System.IO.File.OpenRead (this.Path)) {
+					        header = new PngHeader (input);
+					}
+				}
+				
+				return header;
+			}
+		}
+
+		public System.Collections.ArrayList Chunks {
+			get { return Header.Chunks; }
+		}
+
+
 		public PngFile (string path) : base (path)
 		{
 			this.path = path;
@@ -52,6 +72,7 @@ namespace FSpot.Png {
 						MetadataStore.AddLiteral (sink, "exif:UserComment", text.Text);
 						break;
 					case "Software":
+						MetadataStore.AddLiteral (sink, "xmp:CreatorTool", text.Text);
 						break;
 					case "Title":
 						MetadataStore.AddLiteral (sink, "dc:title", "rdf:Alt", new Literal (text.Text, "x-default", null));
@@ -102,18 +123,6 @@ namespace FSpot.Png {
 			}
 		}
 
-		public System.Collections.ArrayList Chunks {
-			get {
-				if (chunk_list == null) {
-					using (System.IO.Stream input = System.IO.File.OpenRead (this.Path)) {
-						Load (input);
-					}
-				}
-				
-				return chunk_list;
-			}
-		}
-
 		public class ZtxtChunk : TextChunk {
 			//public static string Name = "zTXt";
 
@@ -135,7 +144,27 @@ namespace FSpot.Png {
 				}
 			}
 
-			public ZtxtChunk (string name, byte [] data) : base (name, data) {}
+			public ZtxtChunk (string keyword, string text) : base ()
+			{
+				Name = "zTXt";
+				Compression = 0;
+				this.keyword = keyword;
+			}
+
+			public ZtxtChunk (string name, byte [] data) : base (name, data)
+			{
+			}			
+
+			protected ZtxtChunk ()
+			{
+			}
+
+			public override void SetText (string text)
+			{
+				/* FIXME this is broken */
+				text_data = encoding.GetBytes (text);
+				data = Chunk.Deflate (text_data, 0, text_data.Length);
+			}
 			
 			public override void Load (byte [] data) 
 			{
@@ -174,12 +203,26 @@ namespace FSpot.Png {
 			//public static string Name = "tEXt";
 
 			protected string keyword;
-			protected string text;
 			protected byte [] text_data;
 			protected System.Text.Encoding encoding = Latin1;
 
 			public static System.Text.Encoding Latin1 = System.Text.Encoding.GetEncoding (28591);
-			public TextChunk (string name, byte [] data) : base (name, data) {}
+
+			public TextChunk (string name, byte [] data) : base (name, data) 
+			{
+			}
+
+			protected TextChunk ()
+			{
+			}
+
+
+			public TextChunk (string keyword, string text)
+			{
+				this.Name = "tEXt";
+				this.keyword = keyword;
+				SetText (text);
+			}
 
 			public override void Load (byte [] data)
 			{
@@ -198,13 +241,23 @@ namespace FSpot.Png {
 				}
 			}
 
-			public byte [] TextData 
-			{
+			public byte [] TextData {
 				get {
 					return text_data;
 				}
 			}
 			
+			public virtual void SetText (string text)
+			{
+				text_data = encoding.GetBytes (text);
+
+				byte [] keyword_data = Latin1.GetBytes (keyword);
+				data = new byte [keyword_data.Length + 1 + text_data.Length];
+				System.Array.Copy (keyword_data, 0, data, 0, keyword_data.Length);
+				data [keyword_data.Length] = 0;
+				System.Array.Copy (text_data, 0, data, keyword_data.Length + 1, text_data.Length);
+			}
+
 			public string Text {
 				get {
 					return encoding.GetString (text_data, 0, text_data.Length);
@@ -270,10 +323,63 @@ namespace FSpot.Png {
 				}
 			}
 
+			public override void SetText (string text)
+			{
+				byte [] raw = System.Text.Encoding.UTF8.GetBytes (text);
+				SetText (raw);
+			}
+
+			public void SetText (byte [] raw)
+			{
+				MemoryStream stream = new MemoryStream ();
+				byte [] tmp;
+
+				text_data = raw;
+
+				tmp = Latin1.GetBytes (keyword);
+				stream.Write (tmp, 0, tmp.Length);
+				stream.WriteByte (0);
+
+				stream.WriteByte ((byte)(compressed ? 1 : 0));
+				stream.WriteByte (Compression);
+
+				if (Language != null && Language != "") {
+					tmp = Latin1.GetBytes (Language);
+					stream.Write (tmp, 0, tmp.Length);
+				}
+				stream.WriteByte (0);
+
+				if (LocalizedKeyword != null && LocalizedKeyword != "") {
+					tmp = System.Text.Encoding.UTF8.GetBytes (LocalizedKeyword);
+					stream.Write (tmp, 0, tmp.Length);
+				}
+				stream.WriteByte (0);
+				
+				if (compressed) {
+					tmp = Deflate (text_data, 0, text_data.Length);
+					stream.Write (tmp, 0, tmp.Length);
+				} else {
+					stream.Write (text_data, 0, text_data.Length);
+				}
+				this.data = stream.ToArray ();
+			}
+
 			public ItxtChunk (string name, byte [] data) : base (name, data) 
 			{
+				this.Name = name;
 				encoding = System.Text.Encoding.UTF8;
 			}
+
+			public ItxtChunk (string keyword, string language, bool compressed) : base ()
+			{
+				encoding = System.Text.Encoding.UTF8;
+				this.Name = "iTXt";
+				this.keyword = keyword;
+				this.Language = language;
+				this.LocalizedKeyword = "";
+				this.compressed = compressed;
+				this.Compression = 0;
+			}
 		}
 
 		public class TimeChunk : Chunk {
@@ -300,6 +406,12 @@ namespace FSpot.Png {
 			}
 			
 			public TimeChunk (string name, byte [] data) : base (name, data) {}
+			
+			public TimeChunk ()
+			{
+				this.Name = "tIME";
+				this.Time = System.DateTime.Now;
+			}
 		}
 
 		public class StandardRgbChunk : Chunk {
@@ -470,6 +582,69 @@ namespace FSpot.Png {
 			}
 		}
 
+		public class Crc {
+			static uint [] lookup;
+			uint value = 0xffffffff;
+			uint length;
+			System.IO.Stream stream;
+
+			public uint Value {
+				get { return (value ^ 0xffffffff); }
+			}
+
+			public uint Length {
+				get { return length; }
+			}
+
+			static Crc () {
+				lookup = new uint [265];
+				uint c, n;
+				int k;
+				
+				for (n = 0; n < 256; n++) {
+					c = n;
+					for (k = 0; k < 8; k++) {
+						if ((c & 1) != 0)
+							c = 0xedb88320 ^ (c >> 1);
+						else
+							c = c >> 1;
+					}
+					lookup [n] = c;
+				}
+			}
+
+			public Crc ()
+			{
+			}
+
+			public Crc (System.IO.Stream stream)
+			{
+				this.stream = stream;
+			}
+			
+			public void Write (byte [] buffer)
+			{
+				Write (buffer, 0, buffer.Length);
+			}
+
+			public void Write (byte [] buffer, int offset, int len)
+			{
+				for (int i = offset; i < len; i++) 
+					value = lookup [(value ^ buffer[i]) & 0xff] ^ (value >> 8); 
+
+				length += (uint)len;
+
+				if (stream != null)
+					stream.Write (buffer, offset, len);
+			}
+
+			public void WriteSum ()
+			{
+				byte [] data = BitConverter.GetBytes (Value, false);
+				stream.Write (data, 0, data.Length);
+			}
+		}
+
 		public class Chunk {
 			public string Name;
 			protected byte [] data;
@@ -501,13 +676,16 @@ namespace FSpot.Png {
 				name_table ["sRGB"] = typeof (StandardRgbChunk);
 			}
 			
+			protected Chunk ()
+			{
+			}
+			
 			public Chunk (string name, byte [] data) 
 			{
 				this.Name = name;
 				this.data = data;
 				Load (data);
 			}
-
 			
 			protected string GetString  (ref int i, System.Text.Encoding enc) 
 			{
@@ -528,6 +706,17 @@ namespace FSpot.Png {
 			{
 				
 			}
+			
+			public virtual void Save (System.IO.Stream stream)
+			{
+				byte [] name_bytes = System.Text.Encoding.ASCII.GetBytes (Name);
+				byte [] length_bytes = BitConverter.GetBytes ((uint)data.Length, false);
+				stream.Write (length_bytes, 0, length_bytes.Length);
+				Crc crc = new Crc (stream);
+				crc.Write (name_bytes);
+				crc.Write (data);
+				crc.WriteSum ();
+			}
 
 			public bool Critical {
 				get {
@@ -553,10 +742,14 @@ namespace FSpot.Png {
 				}
 			}
 
-			public bool CheckCrc (uint crc)
+			public bool CheckCrc (uint value)
 			{
-				// FIXME implement me
-				return true;
+				byte [] name = System.Text.Encoding.ASCII.GetBytes (Name);
+				Crc crc = new Crc ();
+				crc.Write (name);
+				crc.Write (data);
+
+				return crc.Value == value;
 			}
 
 			public static Chunk Generate (string name, byte [] data)
@@ -581,17 +774,27 @@ namespace FSpot.Png {
 				
 				byte [] buf = new byte [1024];
 				int inflate_length;
-				while ((inflate_length = inflater.Inflate (buf)) > 0) {
+				while ((inflate_length = inflater.Inflate (buf)) > 0)
 					output.Write (buf, 0, inflate_length);
-				}
 				
-				byte [] result = new byte [output.Length];
-				output.Position = 0;
-				output.Read (result, 0, result.Length);
 				output.Close ();
-				return result;
+				return output.ToArray ();
 			}
 
+			public static byte [] Deflate (byte [] input, int offset, int length)
+			{
+				System.IO.MemoryStream output = new System.IO.MemoryStream ();
+				Deflater deflater = new Deflater ();
+				deflater.SetInput (input, offset, length);
+				
+				byte [] buf = new byte [1024];
+				int deflate_length;
+				while ((deflate_length = deflater.Deflate (buf)) > 0)
+					output.Write (buf, 0, deflate_length);
+
+				output.Close ();
+				return output.ToArray ();
+			}
 		}
 
 		public class ChunkInflater {
@@ -949,82 +1152,136 @@ namespace FSpot.Png {
 			return pixbuf;
 		}
 
-		/*
-		public override Gdk.Pixbuf Load ()
-		{
-			return this.GetPixbuf ();
-		}
-		*/
+		private static byte [] magic = new byte [] { 137, 80, 78, 71, 13, 10, 26, 10 };
 
-	        void Load (System.IO.Stream stream)
-		{
-			byte [] heading = new byte [8];
-			stream.Read (heading, 0, heading.Length);
-
-			if (heading [0] != 137 ||
-			    heading [1] != 80 ||
-			    heading [2] != 78 ||
-			    heading [3] != 71 ||
-			    heading [4] != 13 ||
-			    heading [5] != 10 ||
-			    heading [6] != 26 ||
-			    heading [7] != 10)
-			    throw new System.Exception ("Invalid PNG magic number");
-
-			chunk_list = new System.Collections.ArrayList ();
-
-			for (int i = 0; stream.Read (heading, 0, heading.Length) == heading.Length; i++) {
-				uint length = BitConverter.ToUInt32 (heading, 0, false);
-				string name = System.Text.Encoding.ASCII.GetString (heading, 4, 4);
-				byte [] data = new byte [length];
-				if (length > 0)
-					stream.Read (data, 0, data.Length);
-
-				stream.Read (heading, 0, 4);
-				uint crc = BitConverter.ToUInt32 (heading, 0, false);
-
-				Chunk chunk = Chunk.Generate (name, data);
-				if (! chunk.CheckCrc (crc))
-					throw new System.Exception ("chunk crc check failed");
 				
-				//System.Console.Write ("read one {0} {1}", chunk, chunk.Name);
-				chunk_list.Add (chunk);
+		public class PngHeader {
+			ArrayList chunk_list;
+			
+			public ArrayList Chunks { 
+				get { return chunk_list; }
+			}
+			
+			public PngHeader (Stream stream) 
+			{
+				byte [] heading = new byte [8];
+				byte [] crc_data = new byte [4];
+				stream.Read (heading, 0, heading.Length);
+				
+				for (int i = 0; i < heading.Length; i++)
+				if (heading [i] != magic [i])
+					throw new System.Exception ("Invalid PNG magic number");
+				
+				chunk_list = new System.Collections.ArrayList ();
+				
+				for (int i = 0; stream.Read (heading, 0, heading.Length) == heading.Length; i++) {
+					uint length = BitConverter.ToUInt32 (heading, 0, false);
+					string name = System.Text.Encoding.ASCII.GetString (heading, 4, 4);
+					byte [] data = new byte [length];
+					if (length > 0)
+						stream.Read (data, 0, data.Length);
+					
+					stream.Read (crc_data, 0, 4);
+					uint crc = BitConverter.ToUInt32 (crc_data, 0, false);
+					
+					Chunk chunk = Chunk.Generate (name, data);
+					if (! chunk.CheckCrc (crc))
+						throw new System.Exception ("chunk crc check failed");
+					
+					//System.Console.Write ("read one {0} {1}", chunk, chunk.Name);
+					chunk_list.Add (chunk);
+					
+#if false			       
+					if (chunk is TextChunk) {
+						TextChunk text = (TextChunk) chunk;
+						System.Console.Write (" Text Chunk {0} {1}", 
+								      text.Keyword, "", "");
+					}
+					
+					TimeChunk time = chunk as TimeChunk;
+					if (time != null)
+						System.Console.Write(" Time {0}", time.Time);
 
-#if TEST_METADATA				
-				if (chunk is TextChunk) {
-					TextChunk text = (TextChunk) chunk;
-					System.Console.Write (" Text Chunk {0} {1}", 
-							      text.Keyword, "", text.Text);
-				}
-
-				TimeChunk time = chunk as TimeChunk;
-				if (time != null)
-					System.Console.Write(" Time {0}", time.Time);
+					System.Console.WriteLine ("");
 #endif
-				//System.Console.WriteLine ("");
-				
-				if (chunk.Name == "IEND")
-					break;
+					
+					if (chunk.Name == "IEND")
+						break;
+				}
+			}
+
+			internal string LookupText (string keyword)
+			{
+				TextChunk chunk = LookupTextChunk (keyword);
+				if (chunk != null)
+					return chunk.Text;
+
+				return null;
+			}
+			
+			internal TextChunk LookupTextChunk (string keyword)
+			{
+				foreach (Chunk chunk in Chunks) {
+					TextChunk text = chunk as TextChunk;
+					if (text != null && text.Keyword == keyword)
+						return text;
+				}
+				return null;	
+			}
+			
+			internal void Insert (Chunk chunk)
+			{
+				// FIXME The point of this function is to enforce ordering constraints
+				// it obviously isn't complete right now.
+				if (chunk is IhdrChunk)
+				Chunks.Insert (0, chunk);
+				else if (chunk is TextChunk)
+					Chunks.Insert (1, chunk);
+				else
+					throw new System.Exception ("Uknown ordering for chunk");
+			}
+			
+			public void Save (System.IO.Stream stream)
+			{
+				stream.Write (magic, 0, magic.Length);
+				foreach (Chunk chunk in Chunks) {
+					chunk.Save (stream);
+				}
 			}
 		}
 
-		public string LookupText (string keyword)
-		{
-			TextChunk chunk = LookupTextChunk (keyword);
-			if (chunk != null)
-				return chunk.Text;
 
-			return null;
+		public void Save (System.IO.Stream stream)
+		{
+			Header.Save (stream);
 		}
 
-		public TextChunk LookupTextChunk (string keyword)
+		public void Save (string path)
 		{
-			foreach (Chunk chunk in Chunks) {
-				TextChunk text = chunk as TextChunk;
-				if (text != null && text.Keyword == keyword)
-					return text;
+			string  temp_path = path + ".tmp.png";
+			using (System.IO.Stream output = System.IO.File.OpenWrite (temp_path)) {
+				Save (output);
 			}
-			return null;	
+			if (FSpot.Unix.Rename (temp_path, path) < 0) {
+				System.IO.File.Delete (temp_path);
+				throw new System.Exception (System.String.Format ("Unable to rename {0} to {1}", temp_path, path));
+			}
+		}
+
+		public override void Save (Gdk.Pixbuf pixbuf, System.IO.Stream stream)
+		{
+			byte [] buffer = PixbufUtils.Save (pixbuf, "png", null, null);
+			MemoryStream mem = new MemoryStream (buffer);
+			PngHeader converted = new PngHeader (mem);
+
+			/* FIXME we need to update the XMP metadata here */
+			foreach (Chunk c in Chunks) {
+				if (c is TextChunk) {
+					converted.Insert (c);
+				}
+			}
+
+			converted.Save (stream);
 		}
 
 		public override Cms.Profile GetProfile ()
@@ -1087,15 +1344,59 @@ namespace FSpot.Png {
 
 		public override string Description {
 			get {
-				string description = LookupText ("Description");
+				string description = Header.LookupText ("Description");
 
 				if (description != null)
 					return description;
 				else
-					return LookupText ("Comment");
+					return Header.LookupText ("Comment");
 			}
 		}
 
+		public void SetDescription (string description) 
+		{
+			TextChunk text = null;
+			text = Header.LookupTextChunk ("Description");
+			
+			if (text != null)
+				text.SetText (description);
+			else 
+				Header.Insert (new TextChunk ("Description", description));
+		}
+
+		public XmpFile GetXmp ()
+		{
+			TextChunk xmpchunk  = Header.LookupTextChunk ("XML:com.adobe.xmp");
+			if (xmpchunk == null)
+				xmpchunk = Header.LookupTextChunk ("XMP");
+
+			if (xmpchunk == null)
+				return null;
+			
+			using (MemoryStream stream = new MemoryStream (xmpchunk.TextData)) {
+				return new XmpFile (stream);
+			}
+		}
+
+		public void SetXmp (XmpFile xmp)
+		{
+			TextChunk text = null;
+
+			text = Header.LookupTextChunk ("XML:com.adobe.xmp");
+			if (text != null)
+				Chunks.Remove (text);
+			
+			text = Header.LookupTextChunk ("XMP");
+			if (text != null)
+				Chunks.Remove (text);
+
+			ItxtChunk itext = new ItxtChunk ("XML:com.adobe.xmp", "en", false);
+			MemoryStream stream = new MemoryStream ();
+			xmp.Save (stream);
+			itext.SetText (stream.ToArray ());
+			Header.Insert (itext);
+		}
+
 		public override System.DateTime Date {
 			get {
 				// FIXME: we should first try parsing the
@@ -1119,7 +1420,6 @@ namespace FSpot.Png {
 			}
 		}
 
-
 		public static void Main (string [] args) 
 		{
 			System.Collections.ArrayList failed = new System.Collections.ArrayList ();
