@@ -752,7 +752,7 @@ namespace FSpot.Tiff {
 
 		public Header (System.IO.Stream stream)
 		{
-			using (new Timer ("new Tiff.Header")) {
+			//using (new Timer ("new Tiff.Header")) {
 			byte [] data = new byte [8];
 			stream.Read (data, 0, data.Length);
 			if (data [0] == 'M' && data [1] == 'M')
@@ -784,15 +784,15 @@ namespace FSpot.Tiff {
 			
 			//System.Console.WriteLine ("Reading First IFD");
 			Directory = new ImageDirectory (stream, directory_offset, endian); 
-			}
+			//}
 		}
 		
 		
 		public void Select (SemWeb.StatementSink sink)
 		{
-			using (new Timer ("Tiff.Header.Select")) {
+			//using (new Timer ("Tiff.Header.Select")) {
 				SelectDirectory (Directory, sink);
-			}
+			//}
 		}
 
 		public void SelectDirectory (ImageDirectory dir, StatementSink sink)
@@ -1797,13 +1797,13 @@ namespace FSpot.Tiff {
 					e = sub.Directory [0].Lookup (TagId.DateTimeOriginal);
 					
 					if (e != null)
-						return DirectoryEntry.DateTimeFromString (e.StringValue);
+						return DirectoryEntry.DateTimeFromString (e.StringValue).ToUniversalTime ();
 				}
 
 				e = this.Header.Directory.Lookup (TagId.DateTime);
 
 				if (e != null)
-					return DirectoryEntry.DateTimeFromString (e.StringValue);
+					return DirectoryEntry.DateTimeFromString (e.StringValue).ToUniversalTime ();
 				else
 					return base.Date;
 			}
@@ -2055,20 +2055,6 @@ namespace FSpot.Tiff {
 		{
 			return DCRawFile.Load (this.Path, null);
 		}
-
-		public override System.DateTime Date
-		{
-			get {
-				SubdirectoryEntry sub = (SubdirectoryEntry) this.Header.Directory.Lookup (TagId.ExifIfdPointer);
-				DirectoryEntry e = sub.Directory [0].Lookup (TagId.DateTimeOriginal);
-				
-				if (e != null)
-					return DirectoryEntry.DateTimeFromString (e.StringValue);
-				else
-					return base.Date;
-			}
-		}
-
 	}
 }
 
