@@ -217,6 +217,18 @@ class GtkUtil {
 		i.Show ();
 	}
 
+	public static void MakeCheckMenuItem (Gtk.Menu menu, string label, EventHandler e, bool enabled, bool active, bool as_radio)
+	{
+		Gtk.CheckMenuItem i = new Gtk.CheckMenuItem (label);
+		i.Activated += e;
+		i.Sensitive = enabled;
+		i.DrawAsRadio = as_radio;
+		i.Active = active;
+
+		menu.Append(i);
+		i.Show ();
+	}
+
 	public static void MakeMenuSeparator (Gtk.Menu menu)
 	{
 		Gtk.SeparatorMenuItem i = new Gtk.SeparatorMenuItem ();
